@@ -6,6 +6,7 @@ from zope.app.component.hooks import getSite
 
 from zope.component.interfaces import IFactory
 from zope.i18n import translate
+from zope.i18nmessageid.message import Message
 from zope.app.container.constraints import checkFactory
 from zope.app.publisher.interfaces.browser import AddMenu
 
@@ -40,6 +41,13 @@ from Products.CMFPlone import utils
 
 from plone.app.content.browser.folderfactories import _allowedTypes
 
+
+def _safe_unicode(text):
+    if not isinstance(text, unicode):
+        text = unicode(text, 'utf-8', 'ignore')
+    return text
+
+
 class ActionsSubMenuItem(BrowserSubMenuItem):
     implements(IActionsSubMenuItem)
     
@@ -282,12 +290,8 @@ class DisplayMenu(BrowserMenu):
                                      'extra'        : {'id' : id, 'separator' : None, 'class' : ''},
                                      'submenu'      : None,
                                      })
-            # Make sure the title is unicode
-            title = context.Title()
-            if not isinstance(title, unicode):
-                title = unicode(title, 'utf-8', 'ignore')
-                # Display the selected item (i.e. the context)
-            results.append({ 'title'        : _(u'label_item_selected', default=u'Item: ${contentitem}', mapping={'contentitem' : title}),
+            # Display the selected item (i.e. the context)
+            results.append({ 'title'        : _(u'label_item_selected', default=u'Item: ${contentitem}', mapping={'contentitem' : _safe_unicode(context.Title())}),
                              'description'  : '',
                              'action'       : None,
                              'selected'     : True,
@@ -356,9 +360,8 @@ class DisplayMenu(BrowserMenu):
                                          'extra'        : {'id' : '_contextSetDefaultPage', 'separator' : 'actionSeparator', 'class' : ''},
                                          'submenu'      : None,
                                          })                
-            
                 else:
-                    results.append({ 'title'        : _(u'label_item_selected', default=u'Item: ${title}', mapping={'title' : context.Title()}),
+                    results.append({ 'title'        : _(u'label_item_selected', default=u'Item: ${contentitem}', mapping={'title' : _safe_unicode(context.Title())}),
                                      'description'  : '',
                                      'action'       : None,
                                      'selected'     : True,
@@ -400,8 +403,15 @@ class FactoriesSubMenuItem(BrowserSubMenuItem):
         if showConstrainOptions or len(itemsToAdd) > 1:
             return _(u'label_add_new_item', default=u'Add new\u2026')
         elif len(itemsToAdd) == 1:
-            itemTitle = itemsToAdd[0].Title()
-            return _(u'label_add_type', default='Add ${type}', mapping={'type' : itemTitle})
+            title = itemsToAdd[0].Title()
+            if isinstance(itemsToAdd[0].Title(), Message):
+                title = translate(title, context=self.request)
+            else:
+                title = translate(_safe_unicode(title),
+                                  domain='plone',
+                                  context=self.request)
+            return _(u'label_add_type', default='Add ${type}',
+                     mapping={'type' : title})
         else:
             return _(u'label_add_new_item', default=u'Add new\u2026')
     
@@ -463,7 +473,7 @@ class FactoriesSubMenuItem(BrowserSubMenuItem):
         else:
             locallyAllowed = constrain.getLocallyAllowedTypes()
             return [fti for fti in allowed_types if fti.getId() in locallyAllowed]
-                
+
     @memoize
     def _addingToParent(self):
         return (self._addContext().absolute_url() != self.context.absolute_url())
